"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PrintQuestionSummary = exports.PrintProblemsSummary = exports.CreateQuestionInstance = exports.FormatString = exports.FormatProblemsData = exports.FormatQuestionData = exports.GetExistingProblems = void 0;
const promises_1 = require("fs/promises");
const path_1 = require("path");
const turndown_1 = __importDefault(require("turndown"));
const fs = __importStar(require("fs"));
const OUTPATH = "./problems/"; // Define the output folder of the problem instance
const GetExistingProblems = async (problem_folder) => {
    try {
        var result = [];
        if (!fs.existsSync(problem_folder)) {
            console.error(`Folder ${problem_folder} does not exists.`);
            return [];
        }
        const files = await (0, promises_1.readdir)(problem_folder, { withFileTypes: true });
        result = files.map((value) => {
            return value.name.split("-")[0];
        });
        return result;
    }
    catch (error) {
        console.error("Error reading folder: ", error);
        return [];
    }
};
exports.GetExistingProblems = GetExistingProblems;
const FormatQuestionData = (data) => ({
    link: `https://leetcode.com/problems/` + data.question.titleSlug,
    question: {
        isPaidOnly: data.question.isPaidOnly,
        questionId: data.question.questionId,
        questionFrontendId: data.question.questionFrontendId,
        title: data.question.title,
        titleSlug: data.question.titleSlug,
        difficulty: data.question.difficulty,
        content: data.question.content,
        exampleTestcaseList: data.question.exampleTestcaseList,
        topicTags: data.question.topicTags,
        companyTagStats: data.question.companyTagStats,
        hints: data.question.hints,
        solution: data.question.solution,
        codeSnippets: data.question.codeSnippets,
    },
});
exports.FormatQuestionData = FormatQuestionData;
const FormatProblemsData = (data) => ({
    totalQuestions: data.problemsetQuestionList.total,
    count: data.problemsetQuestionList.questions.length,
    problemsetQuestionList: data.problemsetQuestionList.questions,
});
exports.FormatProblemsData = FormatProblemsData;
const SaveHtmlToMarkdown = (path, content) => {
    // Initialize turndown service and convert
    const markdown = (new turndown_1.default()).turndown(content);
    fs.writeFileSync(path, markdown);
};
const FormatString = (template, ...args) => {
    return template.replace(/{(\d+)}/g, (_, index) => args[index] || "");
};
exports.FormatString = FormatString;
const CreateQuestionInstance = (question, output) => {
    // Check if the input data is null
    if (question === null) {
        console.error("Impossible to create problem instance. Input is null.");
        return;
    }
    const question_root_folder = output || OUTPATH;
    // Create also the root folder if it does not exists
    if (!fs.existsSync(question_root_folder)) {
        if (fs.mkdirSync(question_root_folder, { recursive: true }) === undefined) {
            console.error(`Impossible to create root folder: ${question_root_folder}`);
            return;
        }
    }
    // Create the problem folder name and file name
    const question_name = question.question.titleSlug;
    const question_id = question.question.questionFrontendId;
    const question_full_name = question_id.toString().concat("-", question_name);
    const question_folder = (0, path_1.join)(question_root_folder, question_full_name);
    const question_src = (0, path_1.join)(question_folder, "solution.py");
    const question_readme = (0, path_1.join)(question_folder, "README.md");
    const question_html = (0, path_1.join)(question_folder, "index.html");
    const question_tags = question.question.topicTags
        .map((value) => value.name)
        .reduce((prev, curr) => {
        return prev + ", " + curr;
    });
    // Python header with some information about the problem
    const python_header = "\"\"\"\n" +
        "QUESTION TITLE: " + question.question.title + "\n" +
        "QUESTION LINK: " + question.link + "\n" +
        "QUESTION TAGS: " + question_tags + "\n" +
        "QUESTION LEVEL: " + question.question.difficulty + "\n" +
        "\"\"\"";
    // Build the source code that will be written into the python file
    const source_code = "{0}\n\nfrom typing import *\n\n# SOLUTION STARTS\n{1}\n\n# TESTS\n{2}\n";
    // Take the correct code snippet (Python3)
    const code_snippet = question.question.codeSnippets
        .filter((value) => value.lang === 'Python3')
        .map((value) => value.code)[0] + "\n        ...";
    // Define a regular expression to match the function definition 
    const functionRegex = /def\s+(\w+)\s*\(/;
    const match = code_snippet.match(functionRegex);
    const function_name = match[1];
    const tests = question.question.exampleTestcaseList
        .map((value) => {
        return (0, exports.FormatString)("print(Solution().{0}({1}))", function_name, value.replace("\n", ","));
    })
        .reduce((prev, curr) => {
        return prev + "\n" + curr;
    });
    const full_source_code = (0, exports.FormatString)(source_code, python_header, code_snippet, tests);
    // First we need to create the question folder (it should not exist)
    fs.mkdir(question_folder, 0o777, (err) => {
        if (err)
            console.error("Operation not permitted: ", err);
    });
    // Write the README file using the conversion operation
    SaveHtmlToMarkdown(question_readme, question.question.content);
    // Save the source code content into the source file
    fs.writeFileSync(question_src, full_source_code);
    fs.writeFileSync(question_html, question.question.content);
    // Log the result
    console.log("Result written in folder:", question_folder);
};
exports.CreateQuestionInstance = CreateQuestionInstance;
const PrintProblemsSummary = (problems) => {
    console.log("Total Problems:", problems.totalQuestions);
    console.log("Fetched Problems:", problems.count);
    problems.problemsetQuestionList.forEach((value, idx) => {
        const tags = value.topicTags
            .map((value) => value.name)
            .reduce((p, c) => p + ", " + c);
        const format_str = `[${idx}] <ID=${value.questionFrontendId}> (${value.difficulty}) ` +
            `${value.title.toUpperCase()} [${tags}]`;
        console.log(format_str);
    });
};
exports.PrintProblemsSummary = PrintProblemsSummary;
const PrintQuestionSummary = (question) => {
    console.log((0, exports.FormatString)("Last Question: { ID={0}, TitleSlug={1}, Link={2} }", question.question.questionFrontendId, question.question.titleSlug, question.link));
};
exports.PrintQuestionSummary = PrintQuestionSummary;
